@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Mesh TensorFlow Authors.
+# Copyright 2022 The Mesh TensorFlow Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -89,7 +89,7 @@ class SgdOptimizer(Optimizer):
 
   def apply_grad(self, grad, var):
     if grad is None:
-      tf.logging.warning("Gradient is None for variable %s" % var.name)
+      tf.logging.warning("Gradient is None for variable %s", var.name)
       return []
     # It is critical to use assign_sub instead of mtf.assign(var - ...)
     #  for the case of bfloat16 activations, so as to avoid repeatedly rounding
@@ -115,7 +115,7 @@ class MomentumOptimizer(Optimizer):
 
   def apply_grad(self, grad, var):
     if grad is None:
-      tf.logging.warning("Gradient is None for variable %s" % var.name)
+      tf.logging.warning("Gradient is None for variable %s", var.name)
       return []
 
     updates = []
@@ -153,7 +153,7 @@ class AdamWeightDecayOptimizer(Optimizer):
   def apply_grad(self, grad, var):
     """See base class."""
     if grad is None:
-      tf.logging.warning("Gradient is None for variable %s" % var.name)
+      tf.logging.warning("Gradient is None for variable %s", var.name)
       return []
     grad = mtf.to_float(grad)
 
@@ -217,7 +217,10 @@ class AdafactorOptimizer(Optimizer):
                factored=True,
                epsilon1=1e-30,
                epsilon2=1e-3,
-               min_dim_size_to_factor=128):
+               min_dim_size_to_factor=128,
+               stacked_dim_names=None,
+               exclude_from_parameter_scale=None,
+               ):
     """Construct a new Adafactor optimizer.
 
     See class comment.
@@ -234,6 +237,10 @@ class AdafactorOptimizer(Optimizer):
       epsilon2: Regularization constant for parameter scale.
       min_dim_size_to_factor: only factor accumulator if two tensor dimensions
         are at least this size.
+      stacked_dim_names: an optional list of dimension names never to be
+        factored or reduced over.
+      exclude_from_parameter_scale: an optional list of strings of parameter
+        names to exclude from updating proportion to its parameter scale.
 
     Raises:
       ValueError: if absolute_update_scale and relative_update_scale_fn are both
@@ -252,6 +259,8 @@ class AdafactorOptimizer(Optimizer):
     self._epsilon1 = epsilon1
     self._epsilon2 = epsilon2
     self._min_dim_size_to_factor = min_dim_size_to_factor
+    self._stacked_dim_names = stacked_dim_names or []
+    self._exclude_from_parameter_scale = exclude_from_parameter_scale or []
 
   def _factored_dims(self, shape):
     """Should we use a factored second moment estimator.
@@ -267,9 +276,12 @@ class AdafactorOptimizer(Optimizer):
     Returns:
       either a list of 2 Dimensions or None
     """
-    if not self._factored or shape.ndims < 2:
+    if not self._factored:
       return None
-    sorted_dims = sorted(shape.dims, key=lambda d: -d.size)
+    dims = [d for d in shape.dims if d.name not in self._stacked_dim_names]
+    if len(dims) < 2:
+      return None
+    sorted_dims = sorted(dims, key=lambda d: -d.size)
     if sorted_dims[1].size < self._min_dim_size_to_factor:
       return None
     return sorted_dims[:2]
@@ -288,11 +300,14 @@ class AdafactorOptimizer(Optimizer):
     Returns:
       a Scalar
     """
-    return mtf.maximum(reduce_rms(var), self._epsilon2)
+    output_shape = [
+        d for d in var.shape.dims if d.name in self._stacked_dim_names]
+    return mtf.maximum(
+        reduce_rms(var, output_shape=output_shape), self._epsilon2)
 
   def apply_grad(self, grad, var):
     if grad is None:
-      tf.logging.warning("Gradient is None for variable %s" % var.name)
+      tf.logging.warning("Gradient is None for variable %s", var.name)
       return []
     # create slots
     grad = mtf.to_float(grad)
@@ -320,7 +335,8 @@ class AdafactorOptimizer(Optimizer):
       grad_squared = mtf.square(grad) + self._epsilon1
       decay_rate = self._decay_rate
       old_val = mtf.to_float(var.value)
-      if self._multiply_by_parameter_scale:
+      if self._multiply_by_parameter_scale and not any([
+          s in var.name for s in self._exclude_from_parameter_scale]):
         update_scale = self._parameter_scale(old_val) * self._learning_rate
       else:
         update_scale = self._learning_rate
@@ -363,7 +379,7 @@ class AdafactorOptimizer(Optimizer):
       return updates
 
   def _decay_rate_default(self):
-    return adafactor_decay_rate_pow(0.8)
+    return adafactor_decay_rate_pow()
 
   def _learning_rate_default(self, multiply_by_parameter_scale):
     step_num = tf.cast(tf.train.get_or_create_global_step(), tf.float32)
@@ -388,7 +404,7 @@ def adafactor_decay_rate_adam(beta2):
 
 
 @gin.configurable
-def adafactor_decay_rate_pow(exponent, offset=0):
+def adafactor_decay_rate_pow(exponent=0.8, offset=0):
   """Second moment decay rate where memory-length grows as step_num^exponent.
 
   For fine-tuning, you may want to gin-configure offset to equal the starting
@@ -433,8 +449,8 @@ def adafactor_optimizer_from_hparams(hparams, lr):
       factored=hparams.optimizer_adafactor_factored)
 
 
-def reduce_rms(x):
-  return mtf.sqrt(mtf.reduce_mean(mtf.square(x)))
+def reduce_rms(x, **kwargs):
+  return mtf.sqrt(mtf.reduce_mean(mtf.square(x), **kwargs))
 
 
 # Workaround by copying this over
