@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Mesh TensorFlow Authors.
+# Copyright 2022 The Mesh TensorFlow Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,6 +32,18 @@ import tensorflow.compat.v1 as tf
 
 from tensorflow.python.tpu.ops import tpu_ops  # pylint: disable=g-direct-tensorflow-import
 
+tf.flags.DEFINE_integer(
+    "logical_cores_per_chip",
+    default=2,
+    help="Number of logical accelerator cores per chip.")
+
+tf.flags.DEFINE_bool(
+    "fold_xy",
+    default=False,
+    help="On a 3-D torus fold x and y dimensions into one logical dimension.")
+
+FLAGS = tf.flags.FLAGS
+
 
 @gin.configurable
 class SimdMeshImpl(mtf.MeshImpl):
@@ -826,14 +838,18 @@ def physical_shape_3d_from_topology_proto_4d(mesh_shape):
   Returns:
     a list of length 3
   """
-  if len(mesh_shape) != 4 or mesh_shape[2] != 1:
-    raise ValueError("Expected a 4d shape [x, y, 1, core]")
-  return [mesh_shape[1], mesh_shape[0], mesh_shape[3]]
+  if len(mesh_shape) != 4:
+    raise ValueError("Expected a 4d shape [x, y, z, core]")
+  return [mesh_shape[0] *
+          mesh_shape[1], mesh_shape[2], mesh_shape[3]] if FLAGS.fold_xy else [
+              mesh_shape[1] * mesh_shape[2], mesh_shape[0], mesh_shape[3]
+          ]
 
 
 def auto_logical_to_physical_tpu(logical_shape,
                                  physical_shape,
-                                 return_coordinates=False):
+                                 return_coordinates=False,
+                                 device_assignment=None):
   """Set up a mapping from logical to physical cores for TPU.
 
   We will try to set up a mapping so that allreduce operations are relatively
@@ -851,10 +867,15 @@ def auto_logical_to_physical_tpu(logical_shape,
     physical_shape: a list of integers - typically [X, Y, 1, cores]
     return_coordinates: a boolean - return a list of integer lists (coordinates)
        instead of a list of processor indices
+    device_assignment:  the tpu logical to physical device assignment
 
   Returns:
     logical_to_physical: a permutation of range(product(physical_shape)))
   """
+  # Convert physical shape to 3d
+  if len(physical_shape) == 4:
+    physical_shape = physical_shape_3d_from_topology_proto_4d(physical_shape)
+
   tf.logging.info("auto_logical_to_physical_tpu "
                   "logical_shape=%s physical_shape=%s" %
                   (logical_shape, physical_shape))
@@ -872,14 +893,13 @@ def auto_logical_to_physical_tpu(logical_shape,
     if return_coordinates:
       default = [mtf.pnum_to_processor_coordinates(i) for i in default]
     return default
-  if len(physical_shape) == 4 and physical_shape[2] == 1:
-    physical_shape = physical_shape_3d_from_topology_proto_4d(physical_shape)
-  elif len(physical_shape) != 3:
+
+  if len(physical_shape) != 3:
     tf.logging.warning("Unrecognized format for tpu physical shape")
     return _default_value()
   # physical_shape is a triple of rows, cols, cores
   p0, p1, p2 = physical_shape
-  if p2 != 2:
+  if p2 != FLAGS.logical_cores_per_chip:
     return _default_value
   for dimsize in [p0, p1]:
     # if dimsize not a power of 2, give up
@@ -893,8 +913,8 @@ def auto_logical_to_physical_tpu(logical_shape,
     ring = _ring_2d(p0, p1)
     logical_to_physical = []
     for logical_pnum in range(num_cores):
-      core_on_chip = logical_pnum % 2
-      chip_num = logical_pnum // 2
+      core_on_chip = logical_pnum % FLAGS.logical_cores_per_chip
+      chip_num = logical_pnum // FLAGS.logical_cores_per_chip
       i, j = ring[chip_num]
       logical_to_physical.append((i, j, core_on_chip))
   else:
@@ -939,5 +959,14 @@ def auto_logical_to_physical_tpu(logical_shape,
   if return_coordinates:
     return logical_to_physical
   else:
-    return [mtf.processor_coordinates_to_pnum(physical_shape, coord)
-            for coord in logical_to_physical]
+    if FLAGS.logical_cores_per_chip > 1:
+      return [
+          mtf.processor_coordinates_to_pnum(physical_shape, coord)
+          for coord in logical_to_physical
+      ]
+    else:
+      # check device_assignment
+      if device_assignment is None:
+        raise ValueError("physical_shape or device_assignment unset")
+      return mtf.processor_coordinates_to_pnum_map_nd(
+          physical_shape, logical_to_physical, device_assignment)
